#include "CRingBuffer.h"

CRingBuffer::CRingBuffer(void)
{
	buffer = NULL;
	Resize();
}

CRingBuffer::~CRingBuffer(void)
{
	delete[] CRingBuffer::buffer;
}

void CRingBuffer::Resize(int size)
{
	if (buffer != NULL)
	{
		delete[] buffer;
	}

	if(size <= 0)
	{
		size = RINGBUFFER_SIZE_DEFAULT;
	};

	buffersize = size;

	buffer = new float[buffersize];
	memset(buffer, 0, sizeof(float) * buffersize);

	read_pos = 0;
	SetInterval(buffersize / 2);
	return ;
}

void CRingBuffer::SetInterval(int interval)
{
	if((interval <= 0) || ((interval % buffersize) == 0))
	{
		interval = 1;
	}

	write_pos = (read_pos + interval) % buffersize;
	return ;
}

float CRingBuffer::GetValue(int position)
{
	int pos = (read_pos + position) % buffersize;
	if (pos < 0) { pos = buffersize + pos; }

	return buffer[pos];
}

float CRingBuffer::GetValueWithLerp(float position)
{
	float x1, x2, y1, y2, res;

	x1 = floor(position);
	x2 = x1 + 1.0f;

	y1 = (position - x1);
	y2 = (x2 - position);

	res = (y2 * GetValue((int)x1)) + (y1 * GetValue((int)x2));
	return res;
}


void CRingBuffer::SetValue(float value)
{
	buffer[write_pos] = value;
	ForwardReadPosition();
	ForwardWritePosition();
	return ;
}

void CRingBuffer::ForwardReadPosition(int forward)
{
	int pos = (read_pos + forward) % buffersize;
	if (pos < 0) { pos = buffersize - pos; }

	read_pos = pos % buffersize;
	return ;
}

void CRingBuffer::ForwardWritePosition(int forward)
{
	int pos = (write_pos + forward) % buffersize;
	if (pos < 0) { pos = buffersize - pos; }

	write_pos = pos % buffersize;
	return ;
}
